import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/foundation.dart';
import 'sql_service.dart';
import 'dart:io';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:path_provider/path_provider.dart';

class FirebaseService {
  FirebaseService._();

  static Directory? _tempDir;

  ///lấy hình ảnh tử firebase chuyển thành file
  static Future<File?> getImageFile({
    required String storagePath,
  }) async {
    _tempDir ??= await getTemporaryDirectory();
    final fileName = storagePath.split('/').last;
    final file = File('${_tempDir!.path}/$fileName');
    if (storagePath == '') return null;
    if (!file.existsSync()) {
      try {
        await file.create(recursive: true);
        await FirebaseStorage.instance.ref(storagePath).writeToFile(file);
        return file;
      } catch (e) {
        await file.delete(recursive: true);
        return null;
      }
    } else {
      int sizeFile = await file.length();
      if (sizeFile <= 0) {
        await FirebaseStorage.instance.ref(storagePath).writeToFile(file);
        return file;
      }
    }
    return file;
  }

  static Future logout() async {
    try {
      if (FirebaseAuth.instance.currentUser == null) return;
      if (FirebaseAuth.instance.currentUser?.phoneNumber == null) return;
      String uidPhone = FirebaseAuth.instance.currentUser!.uid;
      String phone = FirebaseAuth.instance.currentUser!.phoneNumber!;
      await SQLService.execute('sp_Logout',
          params: {'phone': phone, 'uidPhone': uidPhone});
      await FirebaseAuth.instance.signOut();
    } catch (e) {
      if (kDebugMode) {
        print('logout: ' + e.toString());
      }
    }
  }

  static Future initNotifyToken(String currentToken) async {
    try {
      if (FirebaseAuth.instance.currentUser == null) return;
      if (FirebaseAuth.instance.currentUser?.phoneNumber == null) return;
      FirebaseMessaging messaging = FirebaseMessaging.instance;
      await messaging.getToken().then((token) async {
        if (currentToken == token) return;
        String uidPhone = FirebaseAuth.instance.currentUser!.uid;
        String phone = FirebaseAuth.instance.currentUser!.phoneNumber!;
        await SQLService.execute('sp_FirebaseMessaging', params: {
          'phone': phone,
          'uidPhone': uidPhone,
          'notifyToken': token
        });
      });
    } catch (e) {
      if (kDebugMode) {
        print('login; ' + e.toString());
      }
    }
  }

  //REQUEST PERMISSION NOTIFY
  static Future<bool> initializeFirebaseMessaging() async {
    final FirebaseMessaging firebaseMessaging = FirebaseMessaging.instance;
    var settings = await firebaseMessaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
    if (settings.authorizationStatus == AuthorizationStatus.authorized) {
      if (kDebugMode) {
        print('User granted permission FirebaseMessaging');
      }
      return true;
    } else {
      if (kDebugMode) {
        print('User declined or has not accepted permission FirebaseMessaging');
      }
      return false;
    }
  }
}
