import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/common.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/sql_service.dart';

class ImageByID {
  ImageByID._();

  static const String prefsImage = 'prefImage';

  static const String _splitValue = '[..]';
  static const String _splitData = '[,,]';

  static String getUrlImage(int id) {
    String idImage = id.toString();
    if (Common.mapImage.containsKey(idImage)) {
      Map<String, Object> data =
          Common.mapImage[idImage] as Map<String, Object>;
      String pathKey = data.keys.firstWhere(
        (element) => element.endsWith('_image_path'),
        orElse: () => data.keys.firstWhere(
          (element) => element.endsWith('_path'),
          orElse: () => '',
        ),
      );
      if (pathKey.isEmpty) return Common.DEFAULT_IMAGE_URL;
      return Methods.getString(data, pathKey);
    } else {
      return Common.DEFAULT_IMAGE_URL;
    }
  }

  /// [value] có thể là id ảnh (ID) hoặc tên ảnh (String)
  ///
  /// Ví dụ:
  ///
  /// {@tool snippet}
  /// ```
  /// ImageByID.getDataImage(12)
  ///
  /// ImageByID.getDataImage('logo_8xland.png')
  /// ```
  /// {@end-tool}
  ///
  static Map<String, Object?> getDataImage(dynamic value) {
    String id = value.toString();
    if (Common.mapImage.containsKey(id)) {
      Map<String, Object> data = Common.mapImage[id] as Map<String, Object>;
      return data;
    } else {
      for (Map<String, Object?> data in Common.mapImage.values) {
        String nameKey = data.keys.firstWhere(
          (element) => element.endsWith('_image_name'),
          orElse: () => data.keys.firstWhere(
            (element) => element.endsWith('_image_path'),
            orElse: () => '',
          ),
        );
        String name = Methods.getString(data, nameKey);
        if (nameKey.endsWith('_image_path')) {
          name = name.split('/').last;
        }
        if (name == id) {
          return data;
        }
      }
      return {
        'hinhAnh_image_path': Common.DEFAULT_IMAGE_URL,
      };
    }
  }

  static Future<void> initHinhAnh({
    List<Map<String, Object>>? dtImage,
    bool reload = false,
  }) async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      if (prefs.getStringList(prefsImage) == null || reload) {
        if (dtImage == null || dtImage.isEmpty || reload) {
          var data = await SQLService.execute(
            'sp_getImage',
          );
          dtImage = Methods.getList(data, 'dsHinhAnh');
        }
        if (Common.uid > 0) {
          await SQLService.execute(
            'sp_setHasMapImage',
            params: {'@uid': Common.uid},
          );
        }
        saveImageToLocal(dsImage: dtImage);
        Common.mapImage = {
          for (var value in dtImage) Methods.getString(value, 'ID'): value
        };
      } else {
        loadImageFromLocal();
      }
    } catch (ex) {
      if (kDebugMode) {
        print('ImageByID.initHinhAnh exception: ' + ex.toString());
      }
    }
  }

  static saveImageToLocal({
    required List<Map<String, Object?>> dsImage,
  }) async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      List<String> images = dsImage.map<String>((e) {
        return e.entries.map((e) {
          String value = e.value != null ? e.value.toString() : '';
          return e.key + _splitValue + value;
        }).join(_splitData);
      }).toList();
      await prefs.setStringList(prefsImage, images);
      if (kDebugMode) {
        print('save Images to local success');
      }
    } catch (e) {
      if (kDebugMode) {
        print('ImageByID.saveImageToLocal exception: ' + e.toString());
      }
    }
  }

  static loadImageFromLocal() async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      List<String> listString = prefs.getStringList(prefsImage) ?? [];
      List<Map<String, Object>> listData = List.generate(
        listString.length,
        (index) => {
          for (var value in listString[index].split(_splitData))
            value.split(_splitValue)[0]: value.split(_splitValue)[1]
        },
      );
      Common.mapImage = {
        for (var data in listData) Methods.getString(data, 'ID'): data
      };
      if (kDebugMode) {
        print('load Images from local success');
      }
    } catch (e) {
      if (kDebugMode) {
        print('ImageByID.loadImageFromLocal exception: ' + e.toString());
      }
    }
  }
}
