import 'dart:io';

import 'package:flutter/material.dart';
import 'package:wowit/firebase_service.dart';

class IconFireBase extends StatefulWidget {
  final String storagePath;
  final double? width;
  final double? height;
  final BoxFit? boxFit;
  final Color? color;
  final Widget? widgetError;
  final Widget? widgetLoadding;
  final double scale = 1.0;
  final String? semanticLabel;
  final bool excludeFromSemantics = false;
  final Animation<double>? opacity;
  final BlendMode? colorBlendMode;
  final AlignmentGeometry alignment = Alignment.center;
  final ImageRepeat repeat = ImageRepeat.noRepeat;
  final Rect? centerSlice;
  final bool matchTextDirection = false;
  final bool gaplessPlayback = false;
  final bool isAntiAlias = false;
  final FilterQuality filterQuality = FilterQuality.low;
  final int? cacheWidth;
  final int? cacheHeight;
  final Color? colorWaiting;
  final Color? colorError;

  const IconFireBase(
      {super.key,
      required this.storagePath,
      this.width,
      this.height,
      this.boxFit,
      this.color,
      this.widgetError,
      this.widgetLoadding,
      this.semanticLabel,
      this.opacity,
      this.colorBlendMode,
      this.centerSlice,
      this.cacheWidth,
      this.cacheHeight,
      this.colorWaiting,
      this.colorError});

  @override
  State<IconFireBase> createState() => _FirebaseIconState();
}

class _FirebaseIconState extends State<IconFireBase> {
  File? image;

  @override
  void initState() {
    _getImage();
    super.initState();
  }

  Future<void> _getImage() async {
    await FirebaseService.getImageFile(storagePath: widget.storagePath)
        .then((value) {
      setState(() {
        image = value;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    if (image == null) {
      return widget.widgetError ?? _waitingView(widget.width, widget.height);
    }
    return Image.file(
      image!,
      width: widget.width,
      height: widget.height,
      key: widget.key,
      fit: widget.boxFit,
      color: widget.color,
      scale: widget.scale,
      errorBuilder: (_, __, ___) =>
          widget.widgetError ?? _waitingView(widget.width, widget.height),
      semanticLabel: widget.semanticLabel,
      excludeFromSemantics: widget.excludeFromSemantics,
      opacity: widget.opacity,
      colorBlendMode: widget.colorBlendMode,
      alignment: widget.alignment,
      repeat: widget.repeat,
      centerSlice: widget.centerSlice,
      matchTextDirection: widget.matchTextDirection,
      gaplessPlayback: widget.gaplessPlayback,
      isAntiAlias: widget.isAntiAlias,
      filterQuality: widget.filterQuality,
      cacheWidth: widget.cacheWidth,
      cacheHeight: widget.cacheHeight,
    );
  }

  static Widget _waitingView(
    double? width,
    double? height, {
    Color? colorWaiting,
  }) {
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: colorWaiting ?? const Color(0xFFF5F5F5),
      ),
      width: width,
      height: height,
    );
  }
}
